@@ -1,5 +1,5 @@
 #
-# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/MakeHelper.pm,v 1.37 2007/02/24 14:12:12 kaffeetisch Exp $
+# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Glib/MakeHelper.pm,v 1.40 2007/10/18 16:05:17 kaffeetisch Exp $
 #
 
 package Glib::MakeHelper;
@@ -138,6 +138,27 @@ sub read_source_list_file {
 	return @list;
 }
 
+=item string = Glib::MakeHelper->get_configure_requires_yaml (%module_to_version)
+
+Generates YAML code that lists every module found in I<%module_to_version>
+under the C<configure_requires> key.  This can be used with
+I<ExtUtils::MakeMaker>'s C<EXTRA_META> parameter to specify which modules are
+needed at I<Makefile.PL> time.
+
+=cut
+
+sub get_configure_requires_yaml {
+  shift; # package name
+  my %prereqs = @_;
+
+  my $yaml = "configure_requires:\n";
+  while (my ($module, $version) = each %prereqs) {
+    $yaml .= "   $module: $version\n";
+  }
+
+  return $yaml;
+}
+
 =item string = Glib::MakeHelper->postamble_clean (@files)
 
 Create and return the text of a realclean rule that cleans up after much 
@@ -353,9 +374,12 @@ sub postamble_docs_full {
 
 	#warn "".scalar(@doctypes)." doctype files\n";
 	#warn "".scalar(@xs_files)." xs files\n";
-	
-	$add_types = "add_types (".join(", ",map {"\"$_\""} @doctypes)."); "
-		if @doctypes;
+
+	if (@doctypes) {
+		$add_types = 'add_types ('
+		           . join(', ', map {'"' . quotemeta ($_) . '"'} @doctypes)
+		           . '); '
+	}
 
 	my $docgen_code = ''
 	    . $add_types
@@ -415,7 +439,8 @@ $blib_done
 
 # documentation stuff
 build/doc.pl :: Makefile @xs_files
-	$^X -I \$(INST_LIB) -I \$(INST_ARCHLIB) -MGlib::ParseXSDoc \\
+	\$(NOECHO) \$(ECHO) Parsing XS files...
+	\$(NOECHO) $^X -I \$(INST_LIB) -I \$(INST_ARCHLIB) -MGlib::ParseXSDoc \\
 		-e 'xsdocparse (qw(@xs_files))' > \$@
 
 # passing all of these files through the single podindex file, which is 
@@ -424,17 +449,19 @@ build/doc.pl :: Makefile @xs_files
 @gend_pods :: build/podindex \$(POD_DEPENDS)
 
 build/podindex :: \$(BLIB_DONE) Makefile build/doc.pl
-	$^X -I \$(INST_LIB) -I \$(INST_ARCHLIB) -MGlib::GenPod -M\$(NAME) \\
+	\$(NOECHO) \$(ECHO) Generating POD...
+	\$(NOECHO) $^X -I \$(INST_LIB) -I \$(INST_ARCHLIB) -MGlib::GenPod -M\$(NAME) \\
 		-e '$docgen_code'
 
 \$(INST_LIB)/\$(FULLEXT)/:
 	$^X -MExtUtils::Command -e mkpath \$@
 
 \$(INST_LIB)/\$(FULLEXT)/index.pod :: \$(INST_LIB)/\$(FULLEXT)/ build/podindex
-	$^X -e 'print \"\\n=head1 NAME\\n\\n\$(NAME) API Reference Pod Index\\n\\n=head1 PAGES\\n\\n=over\\n\\n\"' \\
+	\$(NOECHO) \$(ECHO) Creating POD index...
+	\$(NOECHO) $^X -e 'print \"\\n=head1 NAME\\n\\n\$(NAME) API Reference Pod Index\\n\\n=head1 PAGES\\n\\n=over\\n\\n\"' \\
 		> \$(INST_LIB)/\$(FULLEXT)/index.pod
-	$^X -nae 'print \"=item L<\$\$F[1]>\\n\\n\";' < build/podindex >> \$(INST_LIB)/\$(FULLEXT)/index.pod
-	$^X -e 'print \"=back\\n\\n\";' >> \$(INST_LIB)/\$(FULLEXT)/index.pod
+	\$(NOECHO) $^X -nae 'print \"=item L<\$\$F[1]>\\n\\n\";' < build/podindex >> \$(INST_LIB)/\$(FULLEXT)/index.pod
+	\$(NOECHO) $^X -e 'print \"=back\\n\\n\";' >> \$(INST_LIB)/\$(FULLEXT)/index.pod
 "
 }
 
@@ -531,6 +558,32 @@ sub const_cccmd {
 	$inherited;
 }
 
+#
+# And, some black magick to help make learn to shut the hell up.
+#
+
+sub quiet_rule {
+	my $cmds = shift;
+	my @lines = split /\n/, $cmds;
+	foreach (@lines) {
+		if (/NOECHO/) {
+			# already quiet
+		} elsif (/XSUBPP/) {
+			s/^\t/\t\$(NOECHO) \$(ECHO) [ XS \$< ] && /;
+		} elsif (/CCCMD/) {
+			s/^\t/\t\$(NOECHO) \$(ECHO) [ CC \$< ] && /;
+		} elsif (/\bLD\b/) {
+			s/^\t/\t\$(NOECHO) \$(ECHO) [ LD \$@ ] && /;
+		}
+	}
+	return join "\n", @lines;
+}
+
+sub c_o { return quiet_rule (shift->SUPER::c_o (@_)); }
+sub xs_o { return quiet_rule (shift->SUPER::xs_o (@_)); }
+sub xs_c { return quiet_rule (shift->SUPER::xs_c (@_)); }
+sub dynamic_lib { return quiet_rule (shift->SUPER::dynamic_lib (@_)); }
+
 1;
 
 =head1 AUTHOR
